<?php
//
// It all starts here.. 
//
use app\app;
use app\controllers\account;
use app\controllers\admin;
use app\controllers\category;
use app\controllers\cart;
use app\controllers\checkout;
use app\controllers\home;
use app\controllers\lnurlp;
use app\controllers\lost;
use app\controllers\magic_link;
use app\controllers\support;
use app\controllers\transaction;

require_once __DIR__ . '/../vendor/autoload.php';

// Load environment variables from the .env file at project root
Dotenv\Dotenv::createImmutable(__DIR__ . '/../')->load();

// Start the session
app::init_db();
session_start();
session_regenerate_id(true); // prevent session fixation attacks

// prevent session hijack
if (!isset($_SESSION['fingerprint'])) {
    $_SESSION['fingerprint'] = hash('sha256', $_SERVER['REMOTE_ADDR'] . $_SERVER['HTTP_USER_AGENT']);
} else {
    if ($_SESSION['fingerprint'] !== hash('sha256', $_SERVER['REMOTE_ADDR'] . $_SERVER['HTTP_USER_AGENT'])) {
        session_unset();
        session_destroy();
    }
}

// these will be available to use in all twig templates
$defaults = [
    'copyright_year' => date('Y'),
    'session' => $_SESSION,
    'http_host' => $_SERVER['HTTP_HOST'],
    'env' => $_ENV,
    'is_admin' => isset($_SESSION['user_id']) && $_SESSION['user_id'] == 1,
    // uses cookie-js to get the client's preferred theme
    // used to conditionally deliver image assets 
    // or styles based on theme
    'theme' => isset($_COOKIE["theme"]) ? $_COOKIE["theme"] : 'light',
    // set your tailwind colors here for app themeing
    // the idea is to avoid using colors in your templates
    'colors' => require dirname(__DIR__) . '/src/colors.php',
];

// Setup a twig
$loader = new \Twig\Loader\FilesystemLoader(paths: dirname(__DIR__) . '/src/views');
$GLOBALS['twig'] = new \Twig\Environment($loader, [
    //'cache' => dirname(__DIR__) . '/cache',
    'cache' => false,
]);

$route = explode(separator: '?', string: $_SERVER['REQUEST_URI'])[0];
if (str_starts_with(haystack: $route, needle: '/.well-known/lnurlp/')) {
    $route = '/lnurlp';
}

// Combined regex to match multiple dynamic routes in one go
if (preg_match('/^\/(transaction|user|order|product)\/([\w-]+)$/', $route, $matches)) {
    [$full, $type, $id] = $matches;
    $controllers = [
        'transaction' => fn($id) => transaction::view($defaults, $id),
        'user' => fn($id) => users::view($id),
        'order' => fn($id) => orders::view($id),
        'quote' => fn($id) => quotes::view($id),
        'product' => fn($id) => products::view($id),
        'subscription' => fn($id) => subscriptions::view($id),
        'cart' => fn($id) => cart::index($id),
    ];
    
    if (isset($controllers[$type])) {
        $controller = $controllers[$type]($id);
    }
} else {
    $controller = match ($route) {
        '/' => home::index($defaults),
        '/account' => account::index($defaults),
        '/account/profile' => account::profile(),
        '/account/login' => account::login($defaults),
        '/account/email' => account::email(),
        '/account/logout' => account::logout(),
        '/account/returns' => account::returns($defaults),
        '/account/signup' => account::signup($defaults),
        '/account/billing' => account::billing($defaults),
        '/account/orders' => account::orders($defaults),
        '/account/shipping' => account::shipping($defaults),
        '/account/verify' => account::verify($defaults),
        '/admin' => $defaults['is_admin'] ? admin::index($defaults) : lost::index($defaults),
        '/admin/users' => $defaults['is_admin'] ? admin::users($defaults) : lost::index($defaults),
        '/admin/orders' => $defaults['is_admin'] ? admin::orders($defaults) : lost::index($defaults),
        '/admin/emails' => $defaults['is_admin'] ? admin::emails($defaults) : lost::index($defaults),
        '/admin/transactions' => $defaults['is_admin'] ? admin::transactions($defaults) : lost::index($defaults),
        '/admin/transactions/add' => $defaults['is_admin'] ? admin::transactions_add($defaults) : lost::index($defaults),
        '/admin/transactions/reset' => $defaults['is_admin'] ? admin::transactions_reset($defaults) : lost::index($defaults),
        '/admin/returns' => $defaults['is_admin'] ? admin::returns($defaults) : lost::index($defaults),
        '/magic-link' => magic_link::index(),
        '/checkout/confirmed' => checkout::confirmed($defaults),
        '/checkout/review-pay' => checkout::review_pay($defaults),
        '/checkout/shipping-billing' => checkout::shipping_billing($defaults),
        '/support/ask' => support::index($defaults),
        '/support/bitcoin' => support::bitcoin($defaults),
        '/cart' => cart::index($defaults),
        '/lnurlp' => lnurlp::index(),
        // product categories
        '/power-meters' => category::power_meters($defaults),
        default => lost::index($defaults)
    };
};

// Clear alerts after rendering
foreach (['error', 'warning', 'info', 'success'] as $alert) {
    unset($_SESSION[$alert]);
}
